@@ -4,7 +4,7 @@ from click import BaseCommand
 from django.core.management.base import BaseCommand, CommandParser
 
 from django_nats_nkeys.settings import nats_nkeys_settings
-from django_nats_nkeys.services import init_nsc_operator
+from django_nats_nkeys.services import nsc_init_operator
 
 
 class Command(BaseCommand):
